#include "driver-x.h"

DRIVER_X::DRIVER_X()
{
    _setupA = false;
    _setupB = false;
    _iDirectionA = false;
    _iDirectionB = false;
    _cDirectionA = STOP;
    _cDirectionB = STOP;
    _pinINA1 = -1;
    _pinINA2 = -1;
    _pinINB1 = -1;
    _pinINB2 = -1;
    _pinINA1_PWM = false;
    _pinINA2_PWM = false;
    _pinINB1_PWM = false;
    _pinINB2_PWM = false;
    return;
}

void DRIVER_X::init(int INA1, int INA2, int INB1, int INB2)
{
    if( (INA1 >= 0) || (INA2 >= 0))
    {
        _pinINA1 = INA1;
        _pinINA1_PWM = digitalPinHasPWM(INA1);
        _pinINA2 = INA2;
        _pinINA2_PWM = digitalPinHasPWM(INA2);
        setPin(_pinINA1, _pinINA1_PWM);
        setPin(_pinINA2, _pinINA2_PWM);
        _setupA = true;
    }

    if( (INB1 >= 0) || (INB2 >= 0))
    {
        _pinINB1 = INB1;
        _pinINB1_PWM = digitalPinHasPWM(INB1);
        _pinINB2 = INB2;
        _pinINB2_PWM = digitalPinHasPWM(INB2);
        setPin(_pinINB1, _pinINB1_PWM);
        setPin(_pinINB2, _pinINB2_PWM);
        _setupB = true;
    }
}

void DRIVER_X::initA(int INA1, int INA2)
{
    init(INA1, INA2, _pinINB1, _pinINB2);
}

void DRIVER_X::initB(int INB1, int INB2)
{
    init(_pinINA1, _pinINA2,INB1, INB2);
}

int DRIVER_X::testSpeed(int speed)
{
    if(speed < 0) return 0;
    if(speed > __MAX_PWM_RESOLUTION__) return __MAX_PWM_RESOLUTION__;
    return speed;
}

void DRIVER_X::forwardA(int speed)
{
    speed = testSpeed(speed);
    if(!_setupA) return;
    _cDirectionA = FORWARD;
    _speedA = speed;
    if(_iDirectionA)
    {
        _pinINA1_PWM ? analogWrite(_pinINA1, speed) : digitalWrite(_pinINA1, HIGH);
        _pinINA2_PWM ? analogWrite(_pinINA2, 0) : digitalWrite(_pinINA2, LOW);
        return;
    }
    _pinINA1_PWM ? analogWrite(_pinINA1, 0) : digitalWrite(_pinINA1, LOW);
    _pinINA2_PWM ? analogWrite(_pinINA2, speed) : digitalWrite(_pinINA2, HIGH);
    
}

void DRIVER_X::forwardB(int speed)
{
    speed = testSpeed(speed);
    if(!_setupB) return;
    _cDirectionB = FORWARD;
    _speedB = speed;
    if(_iDirectionB)
    {
        _pinINB1_PWM ? analogWrite(_pinINB1, speed) : digitalWrite(_pinINB1, HIGH);
        _pinINB2_PWM ? analogWrite(_pinINB2, 0) : digitalWrite(_pinINB2, LOW);
        return;
    }
    _pinINB1_PWM ? analogWrite(_pinINB1, 0) : digitalWrite(_pinINB1, LOW);
    _pinINB2_PWM ? analogWrite(_pinINB2, speed) : digitalWrite(_pinINB2, HIGH);
    
}

void DRIVER_X::backwardA(int speed)
{
    speed = testSpeed(speed);
    if(!_setupA) return;
    _cDirectionA = BACKWARD;
    _speedA = speed;
    if(_iDirectionA)
    {
        _pinINA1_PWM ? analogWrite(_pinINA1, 0) : digitalWrite(_pinINA1, LOW);
        _pinINA2_PWM ? analogWrite(_pinINA2, speed) : digitalWrite(_pinINA2, HIGH);    
        return;
    }
    _pinINA1_PWM ? analogWrite(_pinINA1, speed) : digitalWrite(_pinINA1, HIGH);
    _pinINA2_PWM ? analogWrite(_pinINA2, 0) : digitalWrite(_pinINA2, LOW);
    
}

void DRIVER_X::backwardB(int speed)
{
    speed = testSpeed(speed);
    if(!_setupB) return;
    _cDirectionB = BACKWARD;
    _speedB = speed;
    if(_iDirectionB)
    {
        _pinINB1_PWM ? analogWrite(_pinINB1, 0) : digitalWrite(_pinINB1, LOW);
        _pinINB2_PWM ? analogWrite(_pinINB2, speed) : digitalWrite(_pinINB2, HIGH);    
        return;
    }
    _pinINB1_PWM ? analogWrite(_pinINB1, speed) : digitalWrite(_pinINB1, HIGH);
    _pinINB2_PWM ? analogWrite(_pinINB2, 0) : digitalWrite(_pinINB2, LOW);
    
}

void DRIVER_X::invertWiringA()
{
    _iDirectionA = !_iDirectionA;
}

void DRIVER_X::invertWiringB()
{
    _iDirectionB = !_iDirectionB;
}

void DRIVER_X::invertA()
{
    if(!_setupA) return;
    switch (_cDirectionA)
    {
    case FORWARD:
        backwardA(_speedA);
        break;
    case BACKWARD:
        forwardA(_speedA);
        break;
    case STOP:
    default:
        stopA();
        break;
    }
}

void DRIVER_X::invertB()
{
    if(!_setupB) return;
    switch (_cDirectionB)
    {
    case FORWARD:
        backwardB(_speedB);
        break;
    case BACKWARD:
        forwardB(_speedB);
        break;
    case STOP:
    default:
        stopB();
        break;
    }
}

void DRIVER_X::stopA()
{
    if(!_setupA) return;
    _pinINA1_PWM ? analogWrite(_pinINA1, 0) : digitalWrite(_pinINA1, LOW);
    _pinINA2_PWM ? analogWrite(_pinINA2, 0) : digitalWrite(_pinINA2, LOW);
}
void DRIVER_X::stopB()
{
    if(!_setupB) return;
    _pinINB1_PWM ? analogWrite(_pinINB1, 0) : digitalWrite(_pinINB1, LOW);
    _pinINB2_PWM ? analogWrite(_pinINB2, 0) : digitalWrite(_pinINB2, LOW);
}