/*
AUTOR: MICROSIDE TECHNOLOGY S.A. DE C.V.
FECHA: MARZO 2023

  driver-x library

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef _DRIVER_X_H_
#define _DRIVER_X_H_

#include <Arduino.h>

//TODO: List pwm resolution for different architectures
#if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega4808__) || defined(__AVR_ATmega328p__)
#define __MAX_PWM_RESOLUTION__ 255
#else
#define __MAX_PWM_RESOLUTION__ 255
#endif

typedef enum{
    FORWARD,
    BACKWARD,
    STOP
} Direction;

class DRIVER_X {
    private:
        int _pinINA1;
        int _pinINA2;
        int _pinINB1;
        int _pinINB2;
        bool _pinINA1_PWM;
        bool _pinINA2_PWM;
        bool _pinINB1_PWM;
        bool _pinINB2_PWM;
        bool _setupA;
        bool _iDirectionA;
        bool _setupB;
        bool _iDirectionB;
        int _speedA;
        int _speedB;
        Direction _cDirectionA;
        Direction _cDirectionB;

        inline void setPin(byte pin, bool hasPWM)
        {
            if(hasPWM)
            {
                analogWrite(pin, 0);
                return;
            }
            pinMode(_pinINA1, OUTPUT);
            digitalWrite(_pinINA1, LOW);
        }
        int testSpeed(int speed);

    public:
        DRIVER_X();
        void init(int INA1, int INA2, int INB1, int INB2);
        void initA(int INA1, int INA2);
        void initB(int INB1, int INB2);
        void invertA();
        void invertB();
        void forwardA(int speed);
        void inline forwardA() { forwardA(__MAX_PWM_RESOLUTION__); }
        void forwardB(int speed);
        void inline forwardB() { forwardB(__MAX_PWM_RESOLUTION__); }
        void backwardA(int speed);
        void inline backwardA() { backwardA(__MAX_PWM_RESOLUTION__); }
        void backwardB(int speed);
        void inline backwardB() { backwardB(__MAX_PWM_RESOLUTION__); }
        void stopA();
        void stopB();
        void invertWiringA();
        void invertWiringB();
};

#endif